package RPM::Source::Transformation::Factory::BundleImport::MetadataSpecUpdater;

use strict;
use warnings;
use Carp;
use File::Basename;
use RPM::Source::BundleImport;
use RPM::Source::BundleImport::Metadata;
use RPM::Source::BundleImport::RedefineMacro qw/redefine_macro/;
use RPM::Source::Transformation::Resource::FlagLoader;
use RPM::Source::Transformation::Resource::MetadataCollector;
use RPM::Source::Transformation::Factory::BundleImport::SetupFlagDB;
use RPM::Source::Transformation::Factory 0.007;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=1;
# TODO: use destrepo distrodb flag; and in Convert too
my $opt_rpm404=1;
our $opt_force_ascii=$opt_rpm404; # due to rpm404
our $opt_metadata_name_match_ignore_case=0;
our $opt_die_on_metadata_name_mismatch=0;
our $opt_die_on_metadata_version_mismatch=0;
our $opt_allow_metadata_version_lag=0;
our $opt_force_update_summary_description;
my $opt_metadata_update_spec=1;

our ($macros_meta_name, $macros_meta_version);

our @LONGOPT=(
    'bundle-metadata-name-match-ignore-case!' => \$opt_metadata_name_match_ignore_case,
    'bundle-metadata-die-on-name-mismatch!' => \$opt_die_on_metadata_name_mismatch,
    'bundle-metadata-die-on-version-mismatch!' => \$opt_die_on_metadata_version_mismatch,
    'bundle-metadata-force-ascii!' => \$opt_force_ascii,
    );

sub print_longopt {
    print "BundleMetadataCheck specific options:\n";
    print "	--no-bundle-metadata-die-on-name-mismatch.\n";
    print "	--no-bundle-metadata-die-on-version-mismatch.\n";
    print "	--no-bundle-metadata-force-ascii.\n";
}

&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('add_summary.txt',FLAG_RESOURCE_LIST_AS_ONE2ONE);

sub register_messages {
    my ($self,$extopt)=@_;
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-bad_metadata_log_and_reject => [],
	# those macros generally aren't used. The ones that use them should explicitly declare them.
	-macros_meta_name => $macros_meta_name ? $macros_meta_name : undef,
	-macros_meta_version => $macros_meta_version ? $macros_meta_version : undef,
	});
}

sub transformations {
    my @TRANSFORMATIONS;
    push @TRANSFORMATIONS,
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_BUNDLE_VERIFY_METADATA,
	    -name => 'metadata_verify',
	    -opt_handler_sub=>\&opt_handler_verify),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_BUNDLE_SETUP,
	    -name => 'metadata_spec_updater',
	    -opt_handler_sub=>\&opt_handler_spec_updater),
	if $opt_metadata_update_spec;

    return @TRANSFORMATIONS;
}

sub opt_handler_verify {
    my (undef,undef,undef,$converter_opt,$resourceobj)=@_;
    my $rejects=$converter_opt->{-bad_metadata_log_and_reject};
    if (@$rejects) {
	foreach my $reject (@$rejects) {
	    &RPM::Source::BundleImport::_log_reject($reject->{-logfile},$converter_opt->{-opt_bundle_name});
	    warn $reject->{-warning};
	}
	return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
    }

    my $metacollector=$resourceobj->metadata_collector;

    if(!$metacollector->has('summary')) {
	my $def_summary=$resourceobj->get_flagdb('add_summary.txt');
	if ($def_summary) {
	    $metacollector->add('summary',
				-priority => MPRIORITY_DEFAULT,
				-value => $def_summary,
				-comment => 'default value found in flags/add_summary.txt.',
		);
	} else {
	    &RPM::Source::BundleImport::_log_reject('reject.MD.no.summary.txt',$converter_opt->{-opt_bundle_name});
	    warn "valid summary not found.\n";
	    return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
	}
    }

    # -----------------------------------------------------------------------------------
    my $metadata_name=$metacollector->getfirst('name');
    my $metadata_version=$metacollector->getfirst('version');

    # ------------- hack to correct tarball name-version splitting ---------------------
    if ($metadata_name and $metadata_version) {
	my $tarname=$converter_opt->{-opt_bundle_name};
	my $tarversion=$converter_opt->{-opt_bundle_version};
	my $expanded_metadata_version=$metadata_version;
	if (defined($tarversion) and $metadata_version ne $tarversion and $metadata_version.'.0' eq $tarversion) {
	    $expanded_metadata_version.='.0';
	}
	# hack around release in tarball (nodejs)
	# Oops: athet-0.0.1-alpha-8.tgz: file name athet-0.0.1-alpha and Name: athet tag does not match :(
	if (defined($tarversion) and
	    ($metadata_name ne $tarname or $expanded_metadata_version ne $tarversion) and
	    "$metadata_name-$expanded_metadata_version" eq "$tarname-$tarversion") {
	    $tarname=$metadata_name;
	    $tarversion=$metadata_version;
	    $converter_opt->{-opt_bundle_name}=$tarname;
	    $converter_opt->{-opt_bundle_version}=$tarversion;
	} elsif ($metadata_name ne $tarname and $metadata_version ne $tarversion and "$metadata_name-$metadata_version" eq "$tarname-$tarversion") {
	    $tarname=$metadata_name;
	    $tarversion=$metadata_version;
	    $converter_opt->{-opt_bundle_name}=$tarname;
	    $converter_opt->{-opt_bundle_version}=$tarversion;
	}
    }
    # ------------- end hack to correct tarball name-version splitting ---------------------

    my $srcfilename=basename($converter_opt->{-opt_bundle_input_file});
    # --- name check ---
    if ($metadata_name) {
	my $tarname=$converter_opt->{-opt_bundle_name};
	if ($metadata_name ne $tarname and not
	    ($opt_metadata_name_match_ignore_case and lc($metadata_name) eq lc($tarname))
	    ) {
	    warn "MetadataCheck: $srcfilename: archive file name $tarname and metadata name: $metadata_name does not match :( \n" if $verbose;
	    if ($opt_die_on_metadata_name_mismatch) {
		&RPM::Source::BundleImport::_log_reject('reject.MD.name_mismatch.txt',$converter_opt->{-opt_bundle_name});
		warn "ERROR: MetadataCheck: $srcfilename: Name $metadata_name as listed in metadata does not match package's.\n";
		return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
	    } else {
		$metadata_name=$tarname;
		$metacollector->add('name',
			-priority => MPRIORITY_OVERRIDE,
			-value => $tarname,
			-comment => 'use bundle_name as override.',
		    );
	    }
	}
    }

    if ($metadata_version) {
	my $tarversion=$converter_opt->{-opt_bundle_version};
	my $cmp=&__compare_pkg_meta_version($tarversion,$metadata_version,$converter_opt,$srcfilename);
	if ($cmp!=0) {
	    warn "check_pkg_meta: $srcfilename: version $tarversion from archive file and metadata version: $metadata_version does not match :( \n" if $verbose;
	    if ($opt_die_on_metadata_version_mismatch and (!$opt_allow_metadata_version_lag or $cmp>0)) {
		&RPM::Source::BundleImport::_log_reject('reject.MD.version_mismatch.txt',$converter_opt->{-opt_bundle_name});
		warn "ERROR: MetadataCheck: $srcfilename: Version $metadata_version as listed in metadata does not match package's.\n";
		return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
	    } else {
		$metacollector->add('version',
			-priority => MPRIORITY_OVERRIDE,
			-value => $tarversion,
			-comment => 'use bundle_version as override.',
		    );
	    }
	}
    }

    # -----------------------------------------------------------------------------------
    # add name from tarball if no name is collected
    if (not defined($metadata_name) and defined $converter_opt->{-opt_bundle_name}) {
	my $tarname=$converter_opt->{-opt_bundle_name};
	$metadata_name=$tarname;
	$metacollector->add('name',
			    -priority => MPRIORITY_DEFAULT,
			    -value => $tarname,
			    -comment => 'use bundle_name as override.',
	    );
    }

    # -----------------------------------------------------------------------------------
    # set version
    my $trimmed_version;
    my $version=$metacollector->getfirst('version');
    if (defined $version) {
	$trimmed_version=$version;
    } else {
	$trimmed_version=$converter_opt->{-opt_bundle_version};
    }
    $trimmed_version=$RPM::Source::BundleImport::package_class->
	trim_package_version($trimmed_version,$converter_opt->{-opt_bundle_name});
    if ($trimmed_version=~/_remove_blib$/ or
	($trimmed_version=~/^[a-zA-Z]/ and $trimmed_version!~/^svn/)
	) {
	&RPM::Source::BundleImport::_log_reject('reject.BT.strange_version.txt',$converter_opt->{-opt_bundle_name});
	warn 'TODO: rejected trimmed_version='.$trimmed_version;
	return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
    }
    $converter_opt->{-version}=$trimmed_version;

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __compare_pkg_meta_version {
    my ($tarversion,$metadata_version,$converter_opt,$srcfilename) =@_;
    if (not defined($tarversion) or $metadata_version eq $tarversion) {
	return 0;
    } else {
	my $trimmed_tarversion=$tarversion;
	$trimmed_tarversion =~ s/-[A-Za-z]+$//;
	$trimmed_tarversion=$RPM::Source::BundleImport::package_class->
	    trim_package_version($trimmed_tarversion,$converter_opt->{-opt_bundle_name});
	if ($metadata_version eq $trimmed_tarversion) {
	    return 0;
	} elsif ($RPM::Source::BundleImport::version_comparator) {
	    my ($compare1,$compare2);
	    eval {
		$compare1 = &$RPM::Source::BundleImport::version_comparator($metadata_version,$tarversion);
		$compare2 = &$RPM::Source::BundleImport::version_comparator($metadata_version,$trimmed_tarversion);
	    };
	    if ($@) {
		warn "check_pkg_meta: with version_comparator: $srcfilename: version comparator failed.\n";
		return 1;
	    } elsif ($compare1==0 || $compare2==0) {
		return 0;
	    } elsif ($compare1<=0 || $compare2<=0) {
		return -1;
	    } else {
		return 1;
	    }
	}
	return 1;
    }
}

sub opt_handler_spec_updater {
    my (undef,$spec,$parent,$converter_opt,$resourceobj)=@_;

    my $metacollector=$resourceobj->metadata_collector;
    my $sectionname='';
    my $pkgsec=$spec->get_section('package',$sectionname);

    my $license=$metacollector->getfirst('license');
    $pkgsec->set_tag('License', $license) if $license;

    my $rpmgroup=$metacollector->getfirst('rpmgroup');
    $pkgsec->set_tag('Group', $rpmgroup) if defined $rpmgroup;

    # set name
    my $name=$metacollector->getfirst('name');
    if (defined $name) {
	my $specname=$pkgsec->get_tag('Name',RAW=>1);
	$pkgsec->set_tag('Name', $name) if ! $specname or $specname eq '%name';
    }

    &__update_url($pkgsec,$metacollector->getfirst('url'));

    &__update_summary_description($spec,$parent,$converter_opt,
	$metacollector->getfirst('summary'),$metacollector->getfirst('description'));


    my $macros_meta_name = $converter_opt->{-macros_meta_name};
    if ($macros_meta_name and $macros_meta_name ne 'name') {
	my $metadata_name=$metacollector->getfirst('_metadata_name');
	&redefine_macro($spec->main_section,$macros_meta_name,$metadata_name);
    }
    my $macros_meta_version = $converter_opt->{-macros_meta_version};
    if ($macros_meta_version and $macros_meta_version ne 'version') {
	my $metadata_version=$metacollector->getfirst('_metadata_version');
	&redefine_macro($spec->main_section,$macros_meta_version,$metadata_version);
    }

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

### TODO: move some code to a trimmer?
sub __update_summary_description {
    my ($spec,$parent,$converter_opt,$meta_summary,$meta_description)=@_;
    if (not $parent or $converter_opt->{-opt_regenerate} or $opt_force_update_summary_description) {
	if ($meta_summary) {
	    ($meta_summary)=split(/\n/,$meta_summary) if $meta_summary=~/\n/s;
	    $meta_summary=substr($meta_summary,0,99).'...' if length($meta_summary)>99;
	    $meta_summary=~s/%/% /g;
	    $meta_summary=&__kill_non_ascii($meta_summary) if $opt_force_ascii;
	    if ($meta_summary!~/[A-Za-z]/m) {
		# warn here ...
	    } else {
		$spec->main_section->set_tag('Summary', $meta_summary);
	    }
	}
	if ($meta_description) {
	    $meta_description=~s/%/%%/g;
	    $meta_description=&__kill_non_ascii($meta_description) if $opt_force_ascii;
	    if ($meta_description!~/^\s*(?:[A-Za-z]|[`"]?\d-?[A-Za-z])/m) { # like, 2D graphics or `This-Module'
		# something like '# TODO' is ignored by rpm 4.0.4
		# warn here ...
		print STDERR "info: MetadataSpecUpdater: description does not start with a char: skipped\n$meta_description\n";
		print STDERR "description:\n$meta_description\n----------------------\n" if $verbose;
	    } else {
		$meta_description=~s/\n*$//s;
		my $descrsection=$spec->get_section('description');
		$descrsection->empty_body();
		$descrsection->push_body($meta_description."\n\n");
	    }
	}
    }
}

sub __update_url {
    my ($section,$urlstring)=@_;
    if ($urlstring) {
	my ($url)=split(/[\s,]+/,$urlstring);
	$url=~s/%/%%/g;
	$section->set_tag('URL', $url) if $url;
    }
}

### TODO: move to Tools::RPM404 and share with Covert
#use POSIX qw(locale_h);
sub __kill_non_ascii {
    my ($line)=@_;
    local $_=$line;
    # some distro use iso or utf8 in description. ALT Linux use ascii only. :(
    #tr/é/e/;
    tr/¡© éöóêåíãøùçõúôûâàïðîëäæýÿñìèòüáþÉÖÓÊÅÍÃØÙÇÕÚÔÛÂÀÏÐÎËÄÆÝßÑÌÈÒÜÁÞ\x98\x99/-.....eooeaiaoucououaaidieaayvnieouabEOOEAIAOUCOUOUAAIDIEAAYVNIEOUAB''''/;
    # exterminatus of the rest of the non-ascii characters
    s/[^[:ascii:]]/./g;  # get rid of non-ASCII characters
    s/[\x00-\x09\x0c\x0e-\x1f]/_/sg;  # get rid of control characters (ESC, ...)
#    s/[\x80-\xff]/./sg;
    # query and save the old locale
#    my $old_locale = setlocale(LC_CTYPE);
#    setlocale(LC_CTYPE, 'C');
#    s/[^[:print:]]/./;
#    # restore the old locale
#    setlocale(LC_CTYPE, $old_locale);
    return $_;
}



1;
