package RPM::Source::Transformation::Factory::BundleImport::SetupSource;

use strict;
use warnings;
use Carp;
use File::Basename;

use RPM::Source::Transformation::Resource::InputBundle;
use RPM::Source::BundleImport::Helpers;
use RPM::Source::Transformation::Factory::BundleImport::RegisterPkgMacros;
use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;
our $opt_die_on_depth=0;

sub register_messages {
    my ($self,$extopt)=@_;
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-opt_bundle_input_file => undef,
	-opt_bundle_source_file => undef,
	-opt_bundle_url => undef,
	-opt_bundle_name => undef,
	-opt_bundle_version => undef,
	-setup_macros_subst_list => [],
	});
}

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_BUNDLE_SETUP,
	-name => 'tarball_update_setup_prep_source',
	-opt_handler_sub=>\&opt_handler),
	;
}

sub opt_handler {
    my (undef,$spec,undef,$converter_opt,$resourceobj)=@_;

    if ($spec->{GEARDIR}) {
	# this transformation is srpm only
	return RPM::Source::TransformContainer::TRANSFORMATION_OK;
    }

    my $file=$converter_opt->{-opt_bundle_source_file};
    $file//=$converter_opt->{-opt_bundle_input_file};
    my $tarball=$resourceobj->bundle_import_input_bundle();

    my @depth=$tarball->depth();
    die "$file: archive depth is ".(scalar @depth).". expected 0 or 1. unsupported layout.\n" if $opt_die_on_depth and @depth>1;

    # STUB; -T is do not unpack source0. possible future use is for specific types of archives.
    my $setupopt='-T -c %name-%version';

    print STDERR "DEBUG: SetupSource: got file=$file\n" if $verbose > 2;
    if ($verbose > 5) {
	require Data::Dumper;
	print STDERR "DEBUG: SetupSource: converter_opt=\n",Data::Dumper::Dumper($converter_opt);
    }
    my $prepsec=$spec->get_section('prep');
    unless ($prepsec) {
	warn "Oops! BundleImport::SetupSource: %prep section not found";
	return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
    }
    if (@depth==0) {
	$setupopt='-c %name-%version';
    } elsif (@depth==1) {
	$setupopt='-n '.&__rpm404_horrible_quote($depth[0]);
	&__subst_macros(\$setupopt,$spec,$converter_opt);
	print STDERR "DEBUG: depth=1 setup options=$setupopt\n" if $verbose>2;
    } elsif (@depth>1) {
	$setupopt='-n '.&__rpm404_horrible_quote($depth[0]);
	&__subst_macros(\$setupopt,$spec,$converter_opt);
	my $depthpath=join('/',@depth[1..$#depth]);
	print STDERR "DEBUG: depth>1 depthpath=$depthpath setup options=$setupopt\n" if $verbose>2;
	$prepsec->push_body(q{# depth is > 1 .. moving up
pushd "}.$depthpath.q{"
  mv -f * }.('../' x $#depth).q{
  mv -f .* }.('../' x $#depth).q{ ||:
popd}."\n");
    }
    # clean up the rpm default
    $setupopt='' if $setupopt eq '-n %{name}-%{version}';

    $prepsec->exclude_body(qr'\%setup');
    $prepsec->unshift_body("\%setup -q $setupopt\n");
    my $source0=basename($file);
    my $sourceurl=$converter_opt->{-opt_bundle_url};
    # if -opt_bundle_source_file is not defined and -opt_bundle_url
    if ($sourceurl) {
	# we can have -opt_bundle_url and/or -opt_bundle_input_file being ruby gem, 
	# but source0 (-opt_bundle_source_file) being tar
	if ($source0 eq basename($sourceurl)) {
	    $source0=$sourceurl;
	    $source0=~s/\?.*$//; # like in http://downloads.sf.net/octave/tsa-4.4.1.tar.gz?download
	    $source0=~s/\%(2[0-9])/%%$1/g; # url: spaces, etc
	}
    }
    &__subst_macros(\$source0,$spec,$converter_opt);
    $spec->main_section->set_tag('Source0',$source0);

    # TODO: ----------------------------------------------------
    # TODO: implement merge in buildreq and use Buildreq always?
    # TODO: ----------------------------------------------------
    # hack: in Buildreq, merge is not implemented yet, so Buildreq can't be used
    # for BundleUpdate; however, multiple unshift_body will pollute spec.
    if ($file=~/\.(zip|jar)$/) {
	if (@RPM::Source::Transformation::Factory::Buildreq::ISA) {
	    $resourceobj->buildreq->add_buildreq_for('for %setup to work','unzip');
	} else {
	    $spec->main_section->verbatim_exclude_and_unshift('BuildRequires: unzip'."\n");
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __subst_macros {
    my ($lineref,$spec,$converter_opt)=@_;
    my $macros_name=$converter_opt->{-macros_pkg_name};
    my $macros_version=$converter_opt->{-macros_pkg_version};
    my $tarname=$converter_opt->{-opt_bundle_name};
    my $tarversion=$converter_opt->{-opt_bundle_version};
    my $version=$converter_opt->{-version};
    # not true for bundle import; those who need it let define it.
    #$macros_name//='name';
    #$macros_version//='version';
    my $subst_macros_listref=$converter_opt->{-setup_macros_subst_list};
    if ($verbose > 3) {
	print STDERR "DEBUG: SetupSource:__SM: line=$$lineref\n";
	print STDERR "DEBUG: SetupSource:__SM: got tarname=$tarname tarversion=$tarversion\n";
	print STDERR "DEBUG: SetupSource:__SM: macros_subst_list=",join(';',@$subst_macros_listref),"\n";
    }
    $$lineref =~ s/\Q$tarname\E/\%\{$macros_name\}/ if defined $tarname and defined $macros_name;
    $$lineref =~ s/\Q$tarversion\E/\%\{$macros_version\}/ if defined $tarversion and defined $macros_version;
    $$lineref =~ s/\Q$version\E/\%\{version\}/ if defined $version;
    my $macros=$spec->macros();
    foreach my $subst (@$subst_macros_listref) {
	my $val=$macros->macro_subst('%'.$subst);
	if ($val) {
	    print STDERR "DEBUG: SetupSource:__SM: subst $val with $subst\n" if $verbose > 2;
	    $$lineref =~ s/\Q$val\E/\%\{$subst\}/;
	}
    }
}

sub __rpm404_horrible_quote {
    my $in=shift;
    $in=~s,',\\'\\\\\\'\\',g; # ' -> \'\\\'\'
    return $in;
}


1;
